; "Falling snow or other brush..." animation generator (falling-snow-or-other-brush-anim.scm)
; Jean-Marc Chatain (jimcha369@gmail.com)
; 2020/04/20
;
; Makes a copy of your image and creates a multi frames
; animation with a falling snow (or other brush) effect
; in all the image or in the selected part.
; The animation may be saved with the gif-plug-in.

(define (script-fu-falling-snow-V10 img		; Image to use for animation
								drawable		; Area of the image to draw, all image or selected parts
								step			; Frames number, brush size and distance between two brushes
								time			; Adjust delay in millisecond between two animated frames
								layers-number	; Adjust the number of flakes layers from smallest to biggest
								difference		; Difference of flakes size between two layer of flakes
								first-size		; The first size of flakes : 1 = smallest to 15 = biggest
								first-angle		; Set the first angle of brush : -180 to 180
								brush-name		; Set the brush name in the brush selector
								brush-color		; Set the brush color in the color selector
								rotation		; Brush rotation : -1 = left, 0 = no, 1 = right
								wind-force		; -3 strong left, 0 no wind, 3 strong right
								blur-level		; Set the blur level applied on snow flakes layer
								rainy-blur		; Set the kinetic blur level for rainy effect on layer
								rainy-angle		; Set the angle of the rainy blur effect
								opacity			; Adjust the opacity of the snow layer from 1 to 100
								mode			; Snow layer mode seen in Gimp choices
								harden-flakes	; Set to duplicate the flakes layers to harden them
								reverse-anim	; Set to reverse the animation during the building
								open-preview	; Set if animation window open or no in the end of script
	)
	(define (rotate-brush brush deviation)		; Change the brush angle adding deviation angle
	(let* ((sum (+ brush deviation)))
		(if (> sum 180) (- sum 360)				; Brush angle value is between -180° and 180°
			(if (< sum -180) (+ sum 360) sum))))

	(let* (	(actual-frame 1)
			(difference (* difference step))			; Set difference by step increment
			(points (cons-array 4 'double))				; Array to set coordinates of points
			(image (car (gimp-image-duplicate img)))
			(source-layer (car (gimp-image-get-active-layer image)))
			(width (car (gimp-image-width image)))
			(height (car (gimp-image-height image)))
			(bound (cdr (gimp-selection-bounds image)))	; Get the bound of the zone : image or selection
			(xmin (car bound))							; Coordinates of zone upper left corner 
			(ymin (cadr bound))
			(xmax (caddr bound))						; Coordinates of zone lower right corner
			(ymax (cadddr bound))
			(zone-width (- xmax xmin))					; Zone width and height can be different than image size
			(zone-height (- ymax ymin))
		)
		(gimp-image-undo-disable image)
		(gimp-context-push)								; To save all context parameters
		(gimp-context-set-brush (car brush-name))
		(gimp-palette-set-foreground brush-color)
		
		(while (<= actual-frame step)
			(let* (
				(actual-layer 0)
				(actual-size (* first-size step))
				(brush-angle first-angle)
				(base-layer (car (gimp-layer-copy source-layer TRUE)))
				(base-name (string-append "Base "							; Base layer name like "Base 1 (40ms)(replace)"
										(number->string actual-frame) " ("	; to create a "ready to save" animation
										(number->string time) "ms) (replace)"))
				(snow-layer (car (gimp-layer-new image width height RGBA-IMAGE "Snow" opacity mode)))
				(double-layer 0)
				)
				(gimp-image-insert-layer image base-layer 0 -1)
				(gimp-item-set-name base-layer base-name)
				(gimp-image-insert-layer image snow-layer 0 -1)
		
				(while (< actual-layer layers-number)
					(let* (
						(actual-row 0)
						(mid-size (/ actual-size 2))
						(scrolling (round (/ actual-size step)))
						(nb-row (+ (truncate (/ zone-height actual-size)) 3))
						(nb-brush (+ (truncate (/ zone-width actual-size)) 2 (abs wind-force)))
						(ypos (+ ymin (- (* (+ actual-frame actual-layer -1) scrolling) mid-size actual-size)))
						(brush-deviation 0)
						)
						(if (<> (+ nb-brush wind-force) 0)									; To avoid "division by zero" error
						(set! brush-deviation (* (/ 360 (+ nb-brush wind-force)) rotation))
						)
						(gimp-context-set-brush-size actual-size)
						
						(while (< actual-row nb-row)
							(let* (
								(actual-pos 0)
								(xpos (+ xmin (* (+ actual-frame actual-layer -1) scrolling wind-force) mid-size))
								)
								(if (> wind-force 0)
								 (set! xpos (- xpos (* actual-size wind-force) actual-size))
								)
								(while (< actual-pos nb-brush)
									(and (> ypos (- ymin mid-size)) (< ypos (+ ymax mid-size))		; Draw only if the brush is in the zone
										(and  (> xpos (- xmin mid-size)) (< xpos (+ xmax mid-size))
											(begin
												(aset points 0 xpos)								; Set (x0, y0) coordinates
												(aset points 1 ypos)
												(aset points 2 xpos)								; Set (x1, y1) coordinates
												(aset points 3 ypos)
												(gimp-context-set-brush-angle brush-angle)
												(gimp-paintbrush-default snow-layer 4 points)
											)
										)
									)
									(set! brush-angle (rotate-brush brush-angle brush-deviation))	; Change brush angle for better snow look
									(set! xpos (+ xpos actual-size))
									(set! actual-pos (+ actual-pos 1))
								)
								(set! ypos (+ ypos actual-size))
								(set! actual-row (+ actual-row 1))
							)
						)
						(set! brush-angle (rotate-brush brush-angle brush-deviation))
						(set! actual-size (+ actual-size difference))
						(set! actual-layer (+ actual-layer 1))
					)
				)
				(if (> blur-level 0)
				(plug-in-gauss RUN-NONINTERACTIVE image snow-layer blur-level blur-level 1))
				(if (> rainy-blur 0)
				(plug-in-mblur RUN-NONINTERACTIVE image snow-layer 0 rainy-blur (- 90 rainy-angle) 0 0))
				(if (= harden-flakes TRUE)
					(begin
						(set! double-layer (car (gimp-layer-copy snow-layer TRUE)))		; Harden flakes layer duplicating hitself
						(gimp-image-insert-layer image double-layer 0 -1)
						(set! snow-layer (car (gimp-image-merge-down image double-layer 0)))
					)
				)
				(set! base-layer (car (gimp-image-merge-down image snow-layer 0)))
				(if (= reverse-anim TRUE)												; Send base layer to bottom to reverse the animation
					(gimp-image-lower-item-to-bottom image base-layer))
				(set! actual-frame (+ actual-frame 1))
			)
		)
		(gimp-image-remove-layer image source-layer)
		(gimp-selection-none image)						; If drawable was a selection remove it
		(gimp-context-pop)								; Restore the previous context parameters
		(gimp-image-undo-enable image)
		(gimp-display-new image)
		(if (= open-preview TRUE)
			(plug-in-animationplay RUN-NONINTERACTIVE image drawable))
	)
)

(script-fu-register "script-fu-falling-snow-V10"
	"Falling snow or other brush V10..."
	"Create a multi-frames image with a falling snow (or other brush) effect in all image or selected parts."
	"Jean-Marc Chatain (jimcha369@gmail.com)"
	"Jean-Marc Chatain"
	"April 2020"
	"RGB* GRAY*"
	SF-IMAGE		"Image" 0
	SF-DRAWABLE		"Drawable" 0
	SF-ADJUSTMENT	_"Frames number"		'(25 5 50 1 1 0 0)
	SF-ADJUSTMENT	_"Delay in ms"			'(50 40 100 10 10 0 0)
	SF-ADJUSTMENT	_"Layers number"		'(5 1 15 1 1 0 0)
	SF-ADJUSTMENT	_"Size difference"		'(1 1 2 1 1 0 0)
	SF-ADJUSTMENT	_"First brush size"		'(3 1 15 1 1 0 0)
	SF-ADJUSTMENT	_"First brush angle"	'(0 -180 180 1 1 0 0)
	SF-BRUSH		_"Brush name"			'("Bristles 03" 100 3 0)
	SF-COLOR		_"Brush color"			'(255 255 255)
	SF-ADJUSTMENT	_"Brush rotation\n-1=left 0=no 1=right"		'(-1 -1 1 1 1 0 0)
	SF-ADJUSTMENT	_"Wind force\n-3-2-1=left 0=no 1 2 3=right"	'(0 -3 3 1 1 0 0)
	SF-ADJUSTMENT	_"Blur level"			'(0 0 5 1 1 0 0)
	SF-ADJUSTMENT	_"Rainy blur"			'(0 0 30 1 1 0 0)
	SF-ADJUSTMENT	_"Rainy angle"			'(0 -90 90 1 1 0 0)
	SF-ADJUSTMENT	_"Opacity"				'(100 1 100 10 10 0 0)
	SF-OPTION		_"Layer mode"
	'(_"Normal legacy" _"Dissolve" _"Behind legacy" _"Multiply legacy" _"Screen legacy" _"Overlay legacy"
	_"Difference legacy" _"Addition legacy" _"Substract legacy" _"Darken only legacy" _"Lighten only legacy"
	_"HSV Hue legacy" _"HSV Saturation legacy" _"HSL Color legacy" _"HSV Value legacy" _"Divide legacy"
	_"Dodge legacy" _"Burn legacy" _"Hardlight legacy" _"Softlight legacy" _"Grain extract legacy"
	_"Grain merge legacy" _"Color erase legacy" _"Overlay" _"LCH Hue" _"LCH Chroma" _"LCH Color" _"LCH Lightness"
	_"Normal" _"Behind" _"Multiply" _"Screen" _"Difference" _"Addition" _"Substract" _"Darken only"
	_"Lighten only" _"HSV Hue" _"HSV Saturation" _"HSL Color" _"HSV Value" _"Divide" _"Dodge" _"Burn"
	_"Hardlight" _"Softlight" _"Grain extract" _"Grain merge" _"Vivid light" _"Pin light" _"Linear light"
	_"Hard mix" _"Exclusion" _"Linear burn" _"Luma darken only" _"Luma lighten only" _"Luminance"
	_"Color erase" _"Erase" _"Merge" _"Split" _"Pass through" _"Replace" _"Anti erase")
	SF-TOGGLE _"Harden the layers of flakes"	FALSE
	SF-TOGGLE _"Reverse the animation"			FALSE
	SF-TOGGLE _"Open animation window at end"	TRUE
)

(script-fu-menu-register "script-fu-falling-snow-V10"
                         "<Image>/Filters/Animation/Animators")
;                         "<Image>/Z Archive/Triage/")
